@@ -1,11 +1,23 @@
 from pydantic import BaseModel, validator, Field, PrivateAttr, Json
 from datetime import timezone
-from typing import Optional, Dict, List, Type, Any, Union, Tuple, Callable, Set
+from typing import (
+    Optional,
+    Dict,
+    List,
+    Type,
+    Any,
+    Union,
+    Tuple,
+    Callable,
+    Set,
+    Hashable,
+)
 import uuid
 import pendulum
 import sqlite3
 import logging
 from collections import Counter, defaultdict
+from collections.abc import Iterable
 from omspy.base import *
 from copy import deepcopy
 from sqlite_utils import Database
@@ -114,7 +126,7 @@ class Order(BaseModel):
     is_multi: bool = False
     last_updated_at: Optional[pendulum.DateTime] = None
     _num_modifications: int = 0
-    _attrs: Tuple[str] = (
+    _attrs: Tuple[str, ...] = (
         "exchange_timestamp",
         "exchange_order_id",
         "status",
@@ -212,11 +224,13 @@ class Order(BaseModel):
         return True if self.parent_id else False
 
     @property
-    def lock(self) -> Union[OrderLock, None]:
+    def lock(self) -> OrderLock:
+        if self._lock is None:
+            self._lock = OrderLock()
         return self._lock
 
     def _get_other_args_from_attribs(
-        self, broker: Any, attribute: str, attribs_to_copy: Optional[Set] = None
+        self, broker: Any, attribute: str, attribs_to_copy: Optional[Iterable] = None
     ) -> Dict[str, str]:
         """
         Get other arguments for the order from attributes
@@ -426,12 +440,22 @@ class Order(BaseModel):
 
 
 class CompoundOrder(BaseModel):
+    """
+    A collection of orders
+    Note
+    ----
+    1) Indexes are added automatically based on the highest key value
+    2) An error is raised if the index value is already used
+    """
+
     broker: Any
     id: Optional[str] = None
     ltp: defaultdict = Field(default_factory=defaultdict)
     orders: List[Order] = Field(default_factory=list)
     connection: Optional[Database] = None
     order_args: Optional[Dict] = None
+    _index: Dict[int, Order] = PrivateAttr(default_factory=defaultdict)
+    _keys: Dict[Hashable, Order] = PrivateAttr(default_factory=defaultdict)
 
     class Config:
         underscore_attrs_are_private = True
@@ -443,6 +467,9 @@ class CompoundOrder(BaseModel):
             self.id = uuid.uuid4().hex
         if self.order_args is None:
             self.order_args = {}
+        if self.orders:
+            for i, o in enumerate(self.orders):
+                self._index[i] = o
 
     @property
     def count(self) -> int:
@@ -466,13 +493,57 @@ class CompoundOrder(BaseModel):
             c.update({symbol: qty})
         return c
 
+    def _get_next_index(self) -> int:
+        idx = max(self._index.keys()) + 1 if self._index else 0
+        return idx
+
+    def _get_by_key(self, key: Hashable) -> Union[Order, None]:
+        return self._keys.get(key)
+
+    def _get_by_index(self, index: int) -> Union[Order, None]:
+        return self._index.get(index)
+
+    def get(self, key: Hashable) -> Union[Order, None]:
+        """
+        Get the order by key or index
+        key
+            index or key
+        returns Order if available or None
+        Note
+        ----
+        1) key is first searched in key and then searched in index
+        2) index starts at zero
+        """
+        order = self._get_by_key(key)
+        if order is None:
+            try:
+                if isinstance(key, (str, int)):
+                    int_key = int(key)
+                    return self._get_by_index(int_key)
+                else:
+                    return None
+            except Exception as e:
+                return None
+        else:
+            return order
+
     def add_order(self, **kwargs) -> Optional[str]:
         kwargs["parent_id"] = self.id
+        index = kwargs.pop("index", self._get_next_index())
+        key = kwargs.pop("key", None)
         if not (kwargs.get("connection")):
             kwargs["connection"] = self.connection
+        if index in self._index:
+            raise IndexError("Order already assigned to this index")
+        if key:
+            if key in self._keys:
+                raise KeyError("Order already assigned to this key")
         order = Order(**kwargs)
-        order.save_to_db()
         self.orders.append(order)
+        self._index[index] = order
+        if key:
+            self._keys[key] = order
+        order.save_to_db()
         return order.id
 
     def _average_price(self, side: str = "buy") -> Dict[str, float]:
@@ -627,7 +698,9 @@ class CompoundOrder(BaseModel):
     def pending_orders(self) -> List[Order]:
         return [order for order in self.orders if order.is_pending]
 
-    def add(self, order: Order) -> Optional[str]:
+    def add(
+        self, order: Order, index: Optional[int] = None, key: Optional[Hashable] = None
+    ) -> Optional[str]:
         """
         Add an order to the existing compound order
         """
@@ -636,8 +709,19 @@ class CompoundOrder(BaseModel):
             order.connection = self.connection
         if not (order.id):
             order.id = uuid.uuid4().hex
-        order.save_to_db()
+        if index is None:
+            index = self._get_next_index()
+        index = int(index)
+        if index in self._index:
+            raise IndexError("Order already assigned to this index")
+        if key:
+            if key in self._keys:
+                raise KeyError("Order already assigned to this key")
         self.orders.append(order)
+        self._index[index] = order
+        if key:
+            self._keys[key] = order
+        order.save_to_db()
         return order.id
 
     def save(self) -> None:
@@ -647,7 +731,6 @@ class CompoundOrder(BaseModel):
         if self.count > 0:
             for order in self.orders:
                 order.save_to_db()
-        pass
 
 
 class OrderStrategy(BaseModel):
