@@ -1,11 +1,18 @@
 """
 General utility functions for conversion and else
 """
-from typing import Dict, Any, List
+from typing import Dict, Any, List, Union, NamedTuple
 from omspy.models import BasicPosition
 from collections import defaultdict
 
 
+class UQty(NamedTuple):
+    q: float
+    f: float
+    p: float
+    c: float
+
+
 def create_basic_positions_from_orders_dict(
     orders: List[Dict],
 ) -> Dict[str, BasicPosition]:
@@ -15,7 +22,7 @@ def create_basic_positions_from_orders_dict(
         list of orders
     returns a dictionary with key being the symbol and values as a BasicPosition model
     """
-    dct = {}
+    dct: Dict[str, BasicPosition] = {}
     for order in orders:
         symbol = order.get("symbol")
         if symbol:
@@ -25,7 +32,7 @@ def create_basic_positions_from_orders_dict(
                 order.get("trigger_price", 0),
             )
             quantity = abs(order.get("quantity", 0))
-            side = order.get("side").lower()
+            side = order["side"].lower()
             if symbol not in dct.keys():
                 # Create symbol if it doesn't exist
                 dct[symbol] = BasicPosition(symbol=symbol)
@@ -99,6 +106,42 @@ def stop_loss_step_decimal(
     """
     step = abs(step)
     m = int(price / step)
-    val = (m + 1) * step if side == "S" else (m * step) - 1
+    val = (m + 1.0) * step if side == "S" else (m * step) - 1.0
     val = val + 1 - dec if side == "S" else val + dec
     return val
+
+
+def update_quantity(q: float, f: float, p: float, c: float) -> UQty:
+    """
+    Update filled,pending and canceled quantity based on the given data
+    q
+        actual quantity orders
+    f
+        filled quantity
+    p
+        pending quantity
+    c
+        canceled/rejected quantity
+    returns a named tuple containing the calculated values
+    Note
+    -----
+    1) q is taken to be the actual quantity
+    2) formula used is **quantity = filled_quantity + pending_quantiy - canceled quantity**
+    3) canceled quantity is given preference over other quantities, filled quantity
+    the next and pending the last priority
+    4) if filled or canceled quantity is greater than given quantity, it is set equal
+    to the quantity and all other quantities are discarded
+    """
+    if c > 0:
+        c = min(c, q)
+        f = q - c
+        p = q - c - f
+    elif f > 0:
+        f = min(f, q)
+        p = q - f
+    elif p > 0:
+        p = min(p, q)
+        f = q - p
+    else:
+        p = q - p
+    return UQty(q, f, p, c)
